<?php
	// MyBB SSO Client Integration plugin code.
	// (C) 2018 CubicleSoft.  All Rights Reserved.
	//
	// This plugin is free to use with the MyBB forum software package.
	// Implements full MyBB SSO client support for the SSO server here:  http://barebonescms.com/documentation/sso/
	// Creative Commons License:  http://creativecommons.org/licenses/by-nc-nd/3.0/us
	// Developed by Thomas Hruska, CubicleSoft Core.

	// Disallow direct access to this file for security reasons.
	if (!defined("IN_MYBB"))
	{
		die("Direct initialization of this file is not allowed.<br /><br />Please make sure IN_MYBB is defined.");
	}

	function SSOClient_info()
	{
		global $lang;

		$lang->load("SSOClient_plugin_admin");

		return Array(
			"name" => $lang->SSOClient_plugin_Name,
			"description" => $lang->SSOClient_plugin_Desc,
			"website" => "http://mods.mybboard.net/view/sso-client-integration",
			"author" => $lang->SSOClient_plugin_Author,
			"authorsite" => "http://www.cubiclesoft.com/",
			"version" => "2.5",
			"guid" => "d5dbb4a3993e61526ab7438c63d1a53e",
			"compatibility" => "18*"
		);
	}

	function SSOClient_do_login($message = "", $register = false)
	{
		global $lang, $sso_mybb_client;

		if ($message == "" && $sso_mybb_client->LoggedIn())  $sso_mybb_client->Logout();
		$extra = array();
		if (isset($_REQUEST["sso_impersonate"]) && is_string($_REQUEST["sso_impersonate"]))  $extra["sso_impersonate"] = $_REQUEST["sso_impersonate"];
		else if (isset($_REQUEST["sso_remote_id"]) && is_string($_REQUEST["sso_remote_id"]))
		{
			$extra["sso_provider"] = "sso_remote";
			$extra["sso_remote_id"] = $_REQUEST["sso_remote_id"];
		}

		// Attempt to route the request for just the Generic Login provider if the user is trying to register.
		if ($register)  $extra["sso_login_action"] = "signup";

		$sso_mybb_client->Login((isset($lang) ? $lang->settings["htmllang"] : ""), $message, $extra);
	}

	// SSO client integration has to happen VERY early on in order to correctly handle returning data from the SSO server.
	// Plugins aren't loaded during upgrades, so the original sign in system takes over.
	// Ignore processing if MYBB_ADMIN_DIR is defined since the user is just viewing the inactive plugins page.
	global $done_shutdown, $mybb;
	$done_shutdown = true;
	if (!defined("IN_UPGRADE") && !defined("MYBB_ADMIN_DIR") && file_exists(MYBB_ROOT . "inc/plugins/sso_client/config.php"))
	{
		// Trick the Admin CP into initiating a login through the SSO server via a vomit-worthy hack.
		if (defined("IN_ADMINCP"))
		{
			$mybb->settings["orig_cpstyle"] = $mybb->settings["cpstyle"];
			$mybb->settings["cpstyle"] = "sso_client";
		}

		// Load the SSO client.  The Admin CP can optionally use a separate SSO client with a different API key.
		if (defined("IN_ADMINCP") && file_exists(MYBB_ROOT . "inc/plugins/admin_sso_client/config.php"))  require_once MYBB_ROOT . "inc/plugins/admin_sso_client/config.php";
		else  require_once MYBB_ROOT . "inc/plugins/sso_client/config.php";

		require_once SSO_CLIENT_ROOT_PATH . "/index.php";

		global $mybb, $sso_mybb_client;

		$sso_mybb_client = new SSO_Client;
		$sso_mybb_client->Init(array("sso_impersonate", "sso_remote_id"));

		// Block all direct login attempts to the Admin CP.
		if (defined("IN_ADMINCP") && isset($mybb->input["do"]) && $mybb->input["do"] == "login")  SSOClient_do_login("Please sign in.");

		// Fix cookie prefix.  Swiped from 'inc/functions.php'.
		$mybb->settings["cookieprefix"] = str_replace(array("\n", "\r", " "), "", $mybb->settings["cookieprefix"]);

		if (!$sso_mybb_client->LoggedIn())
		{
			// Wipe user sign in cookie.
			if (defined("IN_ADMINCP"))
			{
				if (isset($_COOKIE[$mybb->settings["cookieprefix"] . "adminsid"]))
				{
					my_unsetcookie("adminsid");
					my_unsetcookie("acploginattempts");

					unset($_COOKIE[$mybb->settings["cookieprefix"] . "adminsid"]);
					unset($_COOKIE[$mybb->settings["cookieprefix"] . "acploginattempts"]);
				}
			}
			else
			{
				if (isset($_COOKIE[$mybb->settings["cookieprefix"] . "mybbuser"]))
				{
					my_unsetcookie("mybbuser");
					my_unsetcookie("sid");

					unset($_COOKIE[$mybb->settings["cookieprefix"] . "mybbuser"]);
					unset($_COOKIE[$mybb->settings["cookieprefix"] . "sid"]);
				}
			}
		}
		else
		{
			// Process SSO server data.
			if ($sso_mybb_client->UserLoaded())
			{
				global $db;

				$userinfo = array(
					"firstname" => $sso_mybb_client->GetField("first_name", ""),
					"lastname" => $sso_mybb_client->GetField("last_name", ""),
					"username" => $sso_mybb_client->GetField("username"),
					"email" => $sso_mybb_client->GetField("email"),
					"avatar" => $sso_mybb_client->GetField("avatar_url"),
					"website" => $sso_mybb_client->GetField("website_url"),
					"birthday_year" => $sso_mybb_client->GetField("birthday_year"),
					"birthday_month" => $sso_mybb_client->GetField("birthday_month"),
					"birthday_day" => $sso_mybb_client->GetField("birthday_day"),
					"usertitle" => $sso_mybb_client->GetField("mybb_usertitle", ""),
					"gid" => preg_replace('/\s+/', " ", trim(preg_replace('/[^0-9a-z]/', " ", strtolower($sso_mybb_client->GetField("mybb_gid", "2"))))),
				);

				// Look for specific tags.
				if ($sso_mybb_client->HasTag("mybb_mod"))  $userinfo["gid"] = "6";
				if ($sso_mybb_client->HasTag("mybb_supermod"))  $userinfo["gid"] = "3";
				if ($sso_mybb_client->HasTag("mybb_admin"))  $userinfo["gid"] = "4";
				if ($sso_mybb_client->HasTag("mybb_banned"))  $userinfo["gid"] = "7";

				// Site admin 'gid' override.  Always an Administrator.
				if ($sso_mybb_client->IsSiteAdmin())  $userinfo["gid"] = "4";

				// Correct the 'gid'.
				$fix_gid = array(
					"reg" => "2",
					"register" => "2",
					"registered" => "2",
					"supermod" => "3",
					"supermods" => "3",
					"super mod" => "3",
					"super mods" => "3",
					"super moderator" => "3",
					"super moderators" => "3",
					"admin" => "4",
					"admins" => "4",
					"administrator" => "4",
					"administrators" => "4",
					"mod" => "6",
					"mods" => "6",
					"moderator" => "6",
					"moderators" => "6",
					"ban" => "7",
					"banned" => "7",
					"ban user" => "7"
				);

				if (isset($fix_gid[$userinfo["gid"]]))  $userinfo["gid"] = $fix_gid[$userinfo["gid"]];

				$allowed_gids = array(
					"_2" => true,
					"_3" => true,
					"_4" => true,
					"_6" => true,
					"_7" => true,
				);

				if (!isset($allowed_gids["_" . $userinfo["gid"]]))  $userinfo["gid"] = "2";

				// Check banned information that is controlled by the SSO server.
				if ($userinfo["gid"] != "2" && $userinfo["email"] !== false && is_banned_email($userinfo["email"]))  SSOClient_do_login("The forum administrator has banned your e-mail address.  If possible, change the e-mail address associated with your sign in and try again.");

				// Create the user account if it doesn't exist.
				$query = $db->simple_select("p_sso_client", "id, uid", "id = '" . $db->escape_string($sso_mybb_client->GetUserID()) . "'");
				$row = $db->fetch_array($query);
				if (isset($row) && $row !== false)
				{
					// Override MyBB admin and task user account deletions.
					// Account removal should happen at the SSO server level.
					$query = $db->simple_select("users", "uid, loginkey", "uid = '" . $db->escape_string($row["uid"]) . "'");
					$row2 = $db->fetch_array($query);
					if (!isset($row2) || $row2 === false)
					{
						$db->delete_query("p_sso_client", "id = '" . $db->escape_string($sso_mybb_client->GetUserID()) . "'");
						unset($row);
					}
				}

				// Attempt to link the accounts by e-mail address.
				// Useful for existing forum installations or linking two different provider logins.
				if ((!isset($row) || $row === false) && $userinfo["email"] !== false && $userinfo["email"] != "")
				{
					$query = $db->simple_select("users", "uid", "LOWER(email) = '" . $db->escape_string(my_strtolower($userinfo["email"])) . "'");
					$uid = $db->fetch_field($query, "uid");
					if (isset($uid) && $uid > 0)
					{
						$insertarray = array(
							"id" => $db->escape_string($sso_mybb_client->GetUserID()),
							"uid" => $db->escape_string($uid),
							"info" => $db->escape_string(serialize($userinfo))
						);
						$db->insert_query("p_sso_client", $insertarray);

						unset($insertarray["info"]);
						$row = $insertarray;
					}
				}

				// Calculate the user's birthday.
				$birthday = "";
				if ($userinfo["birthday_month"] !== false && (int)$userinfo["birthday_month"] > 0 && (int)$userinfo["birthday_month"] < 13 && $userinfo["birthday_day"] !== false && (int)$userinfo["birthday_day"] > 0 && (int)$userinfo["birthday_day"] < 32)
				{
					$birthday = (int)$userinfo["birthday_day"] . "-" . (int)$userinfo["birthday_month"] . "-";

					if ($userinfo["birthday_year"] !== false && $userinfo["birthday_year"] > 1800)  $birthday .= (int)$userinfo["birthday_year"];
				}

				// Create the user account the old-fashioned way.
				if (!isset($row) || $row === false)
				{
					function SSOClient_MakeValidUsername($username)
					{
						// Remove bad characters.
						$username = trim_blank_chrs($username);
						$username = str_replace(array(unichr(160), unichr(173), unichr(0xCA), dec_to_utf8(8238), dec_to_utf8(8237), dec_to_utf8(8203)), array(" ", "-", "", "", "", ""), $username);
						$username = str_replace(array("<", ">", "&", "\\", ";", ","), "", $username);

						$username = trim(preg_replace('/\s+/', " ", $username));

						if ($username == "")  return "";
						if (is_banned_username($username, true))  return "";

						return $username;
					}

					// Calculate the username.
					$username = "";
					if ($userinfo["username"] !== false)  $username = SSOClient_MakeValidUsername($userinfo["username"]);
					if ($username == "" && $userinfo["email"] !== false && $userinfo["email"] != "")
					{
						$pos = strrpos($userinfo["email"], "@");
						if ($pos === false)  $username = SSOClient_MakeValidUsername($userinfo["email"]);
						else  $username = SSOClient_MakeValidUsername(substr($userinfo["email"], 0, $pos));
					}

					if ($username == "" && $userinfo["firstname"] != "" || $userinfo["lastname"] != "")  $username = SSOClient_MakeValidUsername($userinfo["firstname"] . " " . $userinfo["lastname"]);

					if ($username == "")  $username = SSOClient_MakeValidUsername("sso_" . $sso_mybb_client->GetUserID());

					require_once MYBB_ROOT . "inc/functions_user.php";
					$baseusername = $username;
					$num = 1;
					while (username_exists($username))
					{
						$num++;
						$username = $baseusername . " (" . $num . ")";
					}

					// Security:  MyBB's login keys are weak depending on the platform (especially Windows).
					// Use the SSO client random string generator instead.
					if (!class_exists("SSO_CSPRNG"))  require_once SSO_CLIENT_ROOT_PATH . "/support/sso_random.php";
					$rng = new SSO_CSPRNG();
					$loginkey = $rng->GenerateString(50);

					// More or less swiped from 'inc/datahandlers/user.php'.
					// Since passwords are always invalid, these accounts can't be signed into directly unless the lost password feature is used.
					$insertarray = array(
						"username" => $db->escape_string($username),
						"password" => "-",
						"salt" => $rng->GenerateString(8),
						"loginkey" => $db->escape_string($loginkey),
						"email" => $db->escape_string($userinfo["email"] !== false ? $userinfo["email"] : ""),
						"postnum" => "0",
						"threadnum" => "0",
						"avatar" => $db->escape_string($userinfo["avatar"] !== false ? $userinfo["avatar"] : ""),
						"avatartype" => $db->escape_string($userinfo["avatar"] !== false && $userinfo["avatar"] != "" ? "remote" : ""),
						"usergroup" => (int)$userinfo["gid"],
						"additionalgroups" => "",
						"displaygroup" => 0,
						"usertitle" => $db->escape_string($userinfo["usertitle"] !== false ? $userinfo["usertitle"] : ""),
						"regdate" => (int)TIME_NOW,
						"lastactive" => (int)TIME_NOW,
						"lastvisit" => (int)TIME_NOW,
						"website" => $db->escape_string($userinfo["website"] !== false ? htmlspecialchars($userinfo["website"]) : ""),
						"icq" => 0,
						"aim" => "",
						"yahoo" => "",
						"skype" => "",
						"google" => "",
						"birthday" => $db->escape_string($birthday),
						"birthdayprivacy" => "none",
						"signature" => "",
						"allownotices" => 1,
						"hideemail" => 1,
						"subscriptionmethod" => 0,
						"invisible" => 1,
						"receivepms" => 1,
						"pmnotice" => 1,
						"pmnotify" => 1,
						"threadmode" => "linear",
						"showimages" => 1,
						"showvideos" => 1,
						"showsigs" => 1,
						"showavatars" => 1,
						"showquickreply" => 1,
						"showredirect" => 1,
						"sourceeditor" => 0,
						"ppp" => 0,
						"tpp" => 0,
						"daysprune" => 0,
						"dateformat" => 0,
						"timeformat" => 0,
						"timezone" => $mybb->settings["timezoneoffset"],
						"dst" => ($mybb->settings["dstcorrection"] == 1 ? 1 : 0),
						"dstcorrection" => ($mybb->settings["dstcorrection"] == 1 ? 2 : 0),
						"buddylist" => "",
						"ignorelist" => "",
						"style" => 0,
						"away" => 0,
						"awaydate" => 0,
						"returndate" => 0,
						"awayreason" => "",
						"pmfolders" => "",
						"notepad" => "",
						"referrer" => 0,
						"referrals" => 0,
						"reputation" => 0,
						"regip" => $db->escape_binary(get_ip()),
						"lastip" => "",
						"language" => "",
						"timeonline" => 0,
						"showcodebuttons" => 1,
						"totalpms" => 0,
						"unreadpms" => 0,
						"warningpoints" => 0,
						"moderateposts" => 0,
						"moderationtime" => 0,
						"suspendposting" => 0,
						"suspensiontime" => 0,
						"coppauser" => 0,
						"classicpostbit" => 0,
						"receivefrombuddy" => 0,
						"suspendsignature" => 0,
						"suspendsigtime" => 0,
						"loginattempts" => 1,
						"usernotes" => "",
					);

					$uid = $db->insert_query("users", $insertarray);

					$userfields = array(
						"ufid" => $uid
					);

					$query = $db->simple_select("profilefields", "fid");
					while ($field = $db->fetch_array($query))
					{
						if (isset($userfields["fid" . $field["fid"]]))  continue;

						$userfields["fid" . $field["fid"]] = "";
					}

					$db->insert_query("userfields", $userfields, false);

					// Update forum stats.
					update_stats(array("numusers" => "+1"));

					// Link the newly created user with their SSO user ID.
					$insertarray = array(
						"id" => $db->escape_string($sso_mybb_client->GetUserID()),
						"uid" => $db->escape_string($uid),
						"info" => $db->escape_string(serialize($userinfo))
					);
					$db->insert_query("p_sso_client", $insertarray);

					unset($insertarray["info"]);
					$row = $insertarray;
				}

				// Impossible except for weird database errors.
				if (!isset($row) || $row === false)
				{
					global $error_handler;

					$error_handler->trigger("Unable to create a new user.  Possibly due to database corruption.", MYBB_GENERAL);
				}

				// Update the user's information.
				$query = $db->simple_select("users", "uid, loginkey", "uid = '" . $db->escape_string($row["uid"]) . "'");
				$row2 = $db->fetch_array($query);
				if (!isset($row2) || $row2 === false)
				{
					global $error_handler;

					$error_handler->trigger("Unable to get user information from database.  Possibly due to database corruption.", MYBB_GENERAL);
				}
				$sso_mybb_client->SetData("id", $row2["uid"]);
				$sso_mybb_client->SetData("k", $row2["loginkey"], 75);

				$updatearray = array(
					"email" => $db->escape_string($userinfo["email"] !== false ? $userinfo["email"] : ""),
					"avatar" => $db->escape_string($userinfo["avatar"] !== false ? $userinfo["avatar"] : ""),
					"avatartype" => $db->escape_string($userinfo["avatar"] !== false && $userinfo["avatar"] != "" ? "remote" : ""),
					"usergroup" => (int)$userinfo["gid"],
					"usertitle" => $db->escape_string($userinfo["usertitle"] !== false ? $userinfo["usertitle"] : ""),
					"website" => $db->escape_string($userinfo["website"] !== false ? htmlspecialchars($userinfo["website"]) : ""),
					"birthday" => $db->escape_string($birthday),
				);
				$db->update_query("users", $updatearray, "uid = '" . $db->escape_string($row2["uid"]) . "'", 1);

				// Some extra work needs to be done to get a valid admin session set up.
				if (defined("IN_ADMINCP"))
				{
					$adminsid = $sso_mybb_client->GetData("a");
					if ($adminsid === false && $sso_mybb_client->FromSSOServer())
					{
						if (!class_exists("SSO_CSPRNG"))  require_once SSO_CLIENT_ROOT_PATH . "/support/sso_random.php";
						$rng = new SSO_CSPRNG();
						$adminsid = md5($rng->GenerateString(128));

						$useragent = $_SERVER["HTTP_USER_AGENT"];
						if (my_strlen($useragent) > 100)  $useragent = my_substr($useragent, 0, 100);

						$insertarray = array(
							"sid" => $adminsid,
							"uid" => $db->escape_string($row2["uid"]),
							"loginkey" => $db->escape_string($row2["loginkey"]),
							"ip" => $db->escape_binary(my_inet_pton(get_ip())),
							"dateline" => TIME_NOW,
							"lastactive" => TIME_NOW,
							"data" => my_serialize(array()),
							"useragent" => $db->escape_string($useragent),
						);
						$db->insert_query("adminsessions", $insertarray);
						$db->update_query("adminoptions", array("loginattempts" => 0, "loginlockoutexpiry" => 0), "uid = '" . $db->escape_string($row2["uid"]) . "'", 1);

						my_setcookie("adminsid", $adminsid, "", true);
						my_setcookie("acploginattempts", 0);
						$_COOKIE[$mybb->settings["cookieprefix"] . "adminsid"] = $adminsid;
						$_COOKIE[$mybb->settings["cookieprefix"] . "acploginattempts"] = "0";

						// Fix the issue with clicking 'logout' in the admin resulting in an infinite logout loop.
						if (isset($_REQUEST["action"]) && $_REQUEST["action"] == "logout")
						{
							unset($_GET["action"]);
							unset($_POST["action"]);
							unset($_REQUEST["action"]);
						}
					}
					$sso_mybb_client->SetData("a", $adminsid);
				}

				// Send the SSO client browser cookies.
				$sso_mybb_client->SaveUserInfo();

				if (!defined("IN_ADMINCP"))
				{
					// Send just 'mybbuser'.  A session will automagically be created later by MyBB.
					my_setcookie("mybbuser", $row2["uid"] . "_" . $row2["loginkey"], -1, true);
					$_COOKIE[$mybb->settings["cookieprefix"] . "mybbuser"] = $row2["uid"] . "_" . $row2["loginkey"];
				}
			}

			// Check 'mybbuser' information against the encrypted information.
			if ($sso_mybb_client->GetData("id") === false || $sso_mybb_client->GetData("k") === false)
			{
				unset($_COOKIE[$mybb->settings["cookieprefix"] . "mybbuser"]);
				unset($_COOKIE[$mybb->settings["cookieprefix"] . "adminsid"]);
			}
			else if (isset($_COOKIE[$mybb->settings["cookieprefix"] . "mybbuser"]) && $sso_mybb_client->GetData("id") . "_" . $sso_mybb_client->GetData("k") !== $_COOKIE[$mybb->settings["cookieprefix"] . "mybbuser"])
			{
				unset($_COOKIE[$mybb->settings["cookieprefix"] . "mybbuser"]);
			}
			else if ($sso_mybb_client->IsSiteAdmin())
			{
				// Site admins get implicit super admin privileges (no ban, account deletion protection, etc).
				$mybb->config["super_admins"] .= "," . $sso_mybb_client->GetData("id");
			}

			// Check 'adminsid' information against the encrypted information.
			if ($sso_mybb_client->GetData("a") === false)
			{
				unset($_COOKIE[$mybb->settings["cookieprefix"] . "adminsid"]);
			}
			else if (isset($_COOKIE[$mybb->settings["cookieprefix"] . "adminsid"]) && $sso_mybb_client->GetData("a") !== $_COOKIE[$mybb->settings["cookieprefix"] . "adminsid"])
			{
				unset($_COOKIE[$mybb->settings["cookieprefix"] . "adminsid"]);
			}
		}

		// Fix the global $mybb object.
		// These are dangerous low-level modifications to the core object that are likely to not be upgrade compatible.
		$mybb2 = new MyBB;
		$mybb2->parse_cookies();

		$mybb->request_method = $mybb2->request_method;
		$mybb->input = $mybb2->input;
		$mybb->debug_mode = $mybb2->debug_mode;
		$mybb->cookies = $mybb2->cookies;
		unset($mybb2);
	}
	$done_shutdown = false;

	// Global hooks.
	if (!defined("IN_UPGRADE") && !defined("MYBB_ADMIN_DIR") && file_exists(MYBB_ROOT . "inc/plugins/sso_client/config.php"))
	{
		$plugins->add_hook("global_end", "SSOClient_global_end");
		$plugins->add_hook("member_logout_end", "SSOClient_member_logout_end");
	}

	function SSOClient_install()
	{
		global $mybb, $db, $lang, $config;

		// Make sure the SSO client is set up first.
		if (!is_dir(MYBB_ROOT . "inc/plugins/sso_client/"))
		{
			flash_message($lang->SSOClient_install_no_client, "error");
			admin_redirect("index.php?module=config-plugins");
		}

		if (!file_exists(MYBB_ROOT . "inc/plugins/sso_client/config.php"))
		{
			// Swiped cookie calculation logic from admin/index.php to fix the installation issue introduced in MyBB 1.8.
			$cookiepath = get_current_location("", "", true);
			$cookiepath = substr($cookiepath, 0, strrpos($cookiepath, "/{$config['admin_dir']}/")) . "/";

			if (file_exists(MYBB_ROOT . "inc/plugins/sso_client/install.php"))  flash_message($lang->sprintf($lang->SSOClient_install_no_client_config, htmlspecialchars(urlencode($cookiepath))), "error");
			else  flash_message($lang->SSOClient_install_no_client_config_no_installer, "error");

			admin_redirect("index.php?module=config-plugins");
		}

		// Delete existing table (if any).
		$db->drop_table("p_sso_client");

		// A separate table is required for isolation.  This is going to be "fun" to maintain.
		switch ($db->type)
		{
			case "pgsql":
			{
				$db->query("CREATE TABLE `" . TABLE_PREFIX . "p_sso_client` (id bigint not null primary key, uid int not null, info text not null) " . $db->build_create_table_collation() . ";");

				break;
			}
			case "sqlite":
			case "mysql":
			case "mysqli":
			{
				$db->query("CREATE TABLE `" . TABLE_PREFIX . "p_sso_client` (id bigint(20) not null primary key, uid int(10) not null, info mediumtext not null) " . $db->build_create_table_collation() . ";");

				break;
			}
			default:
			{
				flash_message($lang->sprintf($lang->SSOClient_install_unknown_db_type, $db->type), "error");
				admin_redirect("index.php?module=config-plugins");
			}
		}
	}

	function SSOClient_is_installed()
	{
		global $db;

		if (is_dir(MYBB_ROOT . "inc/plugins/sso_client/") && file_exists(MYBB_ROOT . "inc/plugins/sso_client/config.php") && $db->table_exists("p_sso_client"))  return true;

		return false;
	}

	function SSOClient_uninstall()
	{
		global $db;

		// Delete existing table (if any).
		$db->drop_table("p_sso_client");
	}

	function SSOClient_global_end()
	{
		global $mybb, $header, $lang, $sso_mybb_client;

		// Remove the quick login option from the welcome message area.
		$header = str_replace("onclick=\"$('#quick_login').", "onclick=\"return true;  $('#quick_login').", $header);

		if (defined("THIS_SCRIPT") && THIS_SCRIPT == "member.php" && isset($mybb->input["action"]))
		{
			if ($mybb->input["action"] == "coppa_form")
			{
				error_no_permission();
			}

			// Initiate a login on any of the signup actions.
			if ($mybb->input["action"] == "register" || $mybb->input["action"] == "do_register" || $mybb->input["action"] == "activate" || $mybb->input["action"] == "resendactivation" || $mybb->input["action"] == "do_resendactivation" || $mybb->input["action"] == "lostpw" || $mybb->input["action"] == "do_lostpw" || $mybb->input["action"] == "resetpassword" || $mybb->input["action"] == "do_login" || $mybb->input["action"] == "login")
			{
				// Avoid an infinite login loop.
				if ($sso_mybb_client->FromSSOServer())  redirect("index.php", $lang->redirect_loggedin);
				else  SSOClient_do_login("", ($mybb->input["action"] == "register" || $mybb->input["action"] == "do_register"));
			}
		}
	}

	// Let MyBB handle its own logout and, if that succeeds, execute the SSO client logout.
	function SSOClient_member_logout_end()
	{
		global $sso_mybb_client;

		$sso_mybb_client->Logout();
	}
?>